;Sound test for Chip16
;Shendo 2011
;
;Registers ra-rf used for ASCII printing
;r0 - current selection
;r1 - selector X position
;r2 - selector Y position
;r3 - Gamepad 1 state
;r4 - Gamepad 1 state mirror
;r5 - Max offset number
;r6 - Gamepad 1 dpad timeout
;r7 - Constant 0
;r8 - Scratchpad
;
	cls			;Start clean
	ldi r0,0		;First option selected by default
	ldi r1,80		;Selector X position
	ldi r2,88		;Selector Y position
	ldi r5,2		;Constant 2
	ldi r6,0		;Clear PAD1 timeout
	ldi r7,0		;Constant 0
:start				;Start of the program
	vblnk			;Wait for VBlank
	ldm r3,#FFF0		;Poll PAD1 state
	jme r3,r7,clr_pad_tmo	;Clear timeout if no keys are pressed
	jmp continue_exec	;Skip timeout clearing
:clr_pad_tmo
	ldi r6,0		;Clear PAD1 timeout
:continue_exec
	cls			;Clear the screen
	ldi ra,10		;X coordinate
	ldi rb,10		;Y coordinate
	ldi rd,string_data_0	;Pointer to the string0
	call print_string	;Draw string on screen
	ldi ra,10		;X coordinate
	ldi rb,185		;Y coordinate
	ldi rd,string_data_1	;Pointer to the string1
	call print_string	;Draw string on screen
	ldi ra,10		;X coordinate
	ldi rb,200		;Y coordinate
	ldi rd,string_data_2	;Pointer to the string2
	call print_string	;Draw string on screen
	ldi ra,10		;X coordinate
	ldi rb,215		;Y coordinate
	ldi rd,string_data_3	;Pointer to the string3
	call print_string	;Draw string on screen
	ldi ra,10		;X coordinate
	ldi rb,85		;Y coordinate
	ldi rd,string_data_4	;Pointer to the string4
	call print_string	;Draw string on screen
	ldi ra,10		;X coordinate
	ldi rb,100		;Y coordinate
	ldi rd,string_data_5	;Pointer to the string5
	call print_string	;Draw string on screen
	ldi ra,10		;X coordinate
	ldi rb,115		;Y coordinate
	ldi rd,string_data_6	;Pointer to the string6
	call print_string	;Draw string on screen
	mov r2,r0		;Copy current selection
	muli r2,16		;Convert to offset
	addi r2,88		;Get Y coordinate
	spr #0903		;Set 6x9 px sprites
	drw r1,r2,arrow_sprite	;Draw arrow selector
	mov r4,r3		;Get PAD1 state for manipulation
	andi r4,64		;Isolate A key
	jmz skip_playing	;Skip playing if A is not pressed
	call play_snd		;Play selected sound
:skip_playing
	jme r6,r7,move_up	;Check if PAD1 timeout is zero
	subi r6,1		;Decrement PAD1 timeout
	jmz move_up		;No timeout, read keys
	jmp start		;Jump to beginning (new frame)
:move_up			;Check if PAD1 UP is pressed
	mov r4,r3		;Get PAD1 state for manipulation
	andi r4,1		;Isolate UP key
	jmz move_down		;UP key is not pressed, skip
	jme r0,r7,start		;Min offset reached, skip
	subi r0,1		;Decrement current selection
	ldi r6,15		;Set PAD1 timeout
:move_down			;Check if PAD1 UP is pressed
	mov r4,r3		;Get PAD1 state for manipulation
	andi r4,2		;Isolate DOWN key
	jmz start		;DOWN key is not pressed, skip
	jme r0,r5,start		;Max offset reached, skip
	addi r0,1		;Increment current selection
	ldi r6,15		;Set PAD1 timeout
	jmp start		;Draw new frame
:play_snd
	ldi r8,0		;Load value for checking
	jme r0,r8,play_snd1	;First option is selected
	ldi r8,1		;Load value for checking
	jme r0,r8,play_snd2	;Second option is selected
	ldi r8,2		;Load value for checking
	jme r0,r8,play_snd3	;Third option is selected
ret				;This should be unreachable, but it's better to play it safe
:play_snd1
	snd1 200		;Play 500 Hz tone for 200 ms
ret
:play_snd2
	snd2 200		;Play 1000 Hz tone for 200 ms
ret
:play_snd3
	snd3 200		;Play 1500 Hz tone for 200 ms
ret
:print_string			;Print string on screen
	spr #0F05		;Set 10x15 px sprites
	ldi re,font_data	;Pointer to the graphic data
	ldm rf,rd		;Get ascii code
	andi rf,255		;Use only low part of the data
	jmz print_end		;If the char is null stop printing
	subi rf,32		;Convert to char offset
	jmc print_end		;If the char is below zero it's unprintable
	muli rf,75		;Convert char offset to byte offset
	add re,rf		;Get the char sprite address
	drw ra,rb,re		;Draw char on screen
	addi ra,8		;Increase X offset
	addi rd,1		;Point to next char
	jmp print_string	;Print next char
:print_end
ret
:string_data_0
db #53				;"Sound test"
db #6F
db #75
db #6E
db #64
db #20
db #74
db #65
db #73
db #74
db #00
:string_data_1
db #43				;"Controls:"
db #6F
db #6E
db #74
db #72
db #6F
db #6C
db #73
db #3A
db #00
:string_data_2
db #55				;"Up/Down - select frequency"
db #70
db #2F
db #44
db #6F
db #77
db #6E
db #20
db #2D
db #20
db #73
db #65
db #6C
db #65
db #63
db #74
db #20
db #66
db #72
db #65
db #71
db #75
db #65
db #6E
db #63
db #79
db #00
:string_data_3
db #41				;"A - play sound"
db #20
db #2D
db #20
db #70
db #6C
db #61
db #79
db #20
db #73
db #6F
db #75
db #6E
db #64
db #00
:string_data_4
db #35				;"500 Hz"
db #30
db #30
db #20
db #48
db #7A
db #00
:string_data_5
db #31				;"1000 Hz"
db #30
db #30
db #30
db #20
db #48
db #7A
db #00
:string_data_6
db #31				;"1500 Hz"
db #35
db #30
db #30
db #20
db #48
db #7A
db #00
:arrow_sprite
db #00				;"Sprite data for arrow"
db #00
db #30
db #00
db #03
db #30
db #00
db #33
db #30
db #03
db #33
db #30
db #33
db #33
db #30
db #03
db #33
db #30
db #00
db #33
db #30
db #00
db #03
db #30
db #00
db #00
db #30
:font_data
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #FF
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #FF
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #F0
db #00
db #0F
db #FF
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #FF
db #F0
db #00
db #0F
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #00
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #F0
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #00
db #FF
db #F0
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #FF
db #FF
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #F0
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #0F
db #FF
db #00
db #00
db #0F
db #FF
db #FF
db #FF
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #FF
db #00
db #FF
db #00
db #00
db #FF
db #0F
db #FF
db #00
db #00
db #FF
db #0F
db #FF
db #00
db #00
db #FF
db #00
db #FF
db #00
db #00
db #FF
db #F0
db #FF
db #00
db #00
db #FF
db #F0
db #FF
db #00
db #00
db #FF
db #00
db #FF
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #FF
db #00
db #00
db #FF
db #00
db #FF
db #00
db #00
db #FF
db #00
db #FF
db #00
db #FF
db #FF
db #00
db #FF
db #0F
db #F0
db #FF
db #00
db #FF
db #0F
db #F0
db #FF
db #00
db #FF
db #00
db #FF
db #FF
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #FF
db #0F
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #FF
db #00
db #FF
db #00
db #0F
db #FF
db #F0
db #FF
db #00
db #0F
db #F0
db #FF
db #FF
db #00
db #0F
db #F0
db #0F
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #0F
db #F0
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #FF
db #F0
db #00
db #0F
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #0F
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #0F
db #F0
db #F0
db #FF
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #FF
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #FF
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #0F
db #F0
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #FF
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #FF
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #0F
db #F0
db #00
db #00
db #00
db #FF
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #0F
db #FF
db #00
db #0F
db #00
db #FF
db #0F
db #F0
db #FF
db #00
db #F0
db #00
db #FF
db #F0
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
db #00
:font_cache